import dis


class ServerMaker(type):

    def __init__(cls, clsname, bases, clsdict):

        methods = []  # Список методов, используемых в функциях класса LOAD_GLOBAL или LOAD_METHOD.
        for func in clsdict:
            try:
                ret = dis.get_instructions(clsdict[func])  # Проверяю, что функция.

            except TypeError:
                ...
            else:
                # Только в случае если все элементы - функции, разбираем используемые методы.
                for i in ret:

                    match i.opname:
                        case 'LOAD_GLOBAL':
                            if i.argval not in methods:
                                methods.append(i.argval)

        if any(i for i in ['accept', 'listen', 'socket'] if i in methods):
            raise TypeError('В классе обнаружено использование запрещённого метода!')

        match clsname:
            case 'ClientSender':
                socket_fn = 'send_message'
            case 'ClientReader':
                socket_fn = 'get_message'
        if socket_fn not in methods:
            raise TypeError(f'У класса {clsname} отсутствует функция {socket_fn}!')

        super().__init__(clsname, bases, clsdict)


class ClientMaker(type):

    def __init__(cls, clsname, bases, clsdict):
        methods = []  # Список методов, используемых в функциях класса (LOAD_GLOBAL, LOAD_METHOD).
        attrs = []  # Список аргументов (LOAD_ATTR).
        for func in clsdict:
            try:
                ret = dis.get_instructions(clsdict[func])  # Проверяю, что функция.
            except TypeError:
                ...
            else:
                # Только в случае если все элементы - функции, разбираем используемые методы.
                for i in ret:
                    match i:
                        case 'LOAD_GLOBAL' | 'LOAD_METHOD':
                            if i.argval not in methods:
                                methods.append(i.argval)
                        case 'LOAD_ATTR':
                            if i.argval not in attrs:
                                attrs.append(i.argval)

            if 'connect' in methods:
                raise TypeError('В серверном классе обнаружено использование запрещённого метода "connect"!')
            if not all(i for i in ['SOCK_STREAM', 'AF_INET'] if i in attrs):
                raise TypeError('Некорректная инициализация сокета.')

            super().__init__(clsname, bases, clsdict)



import logging
import sys

from common.variables import DEFAULT_PORT

# Инициализация логера.
# Метод определения модуля, источника запуска.
if sys.argv[0].find('client') == -1:
    # если не клиент, то сервер!
    logger = logging.getLogger('server')
else:
    # ну, раз не сервер, то клиент
    logger = logging.getLogger('client')


class PortValidator:
    def __set__(self, instance, value):
        if value < 1024 or value > 65535:
            logger.critical(
                f'Номер {value} не является приемлемым номером порта. '
                f'Допустимы номера в диапазоне 1024-65535.')
            raise TypeError('Некорректный номер порта')
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name



"""Unit-тесты клиента"""

import sys
import os
import unittest
sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import RESPONSE, ERROR, USER, ACCOUNT_NAME, TIME, ACTION, PRESENCE
from client import create_presence, process_ans


class TestClass(unittest.TestCase):
    """Kлаcc с тестами"""

    def test_create_presence(self):
        """Тест корректного запроса"""
        test = create_presence()
        test[TIME] = 1665309507.8688447  # При работе программы параметр уникален, для проверки задаю
        # время константой (текущее время на момент написания дом. задания).
        self.assertEqual(test, {ACTION: PRESENCE, TIME: 1665309507.8688447, USER: {ACCOUNT_NAME: 'Guest'}})

    def test_200_ans(self):
        """Тест корректного разбора ответа 200"""
        self.assertEqual(process_ans({RESPONSE: 200}), '200 : Ok!')

    def test_type_200_ans(self):
        """Тест корректного типа данных для ответа 200"""
        self.assertTrue(isinstance(process_ans({RESPONSE: 200}), str), 'Возвращаемый тип данных не строка')
        #  Тест бессмысленный в плане проверки работоспособности конкретной функции,
        #  т.к. у нас будет строка в любом случае, но показывающий возможность проверки истинности выражений.

    def test_400_ans(self):
        """Тест корректного разбора 400"""
        self.assertEqual(process_ans({RESPONSE: 400, ERROR: 'Bad Request'}), '400 : Bad Request')

    def test_no_response(self):
        """Тест исключения без поля RESPONSE"""
        self.assertRaises(ValueError, process_ans, {ERROR: 'Bad Request'})


if __name__ == '__main__':
    unittest.main()



import sys
import os


def main():
    PYTHON_PATH = sys.executable
    file_path = os.getcwd() + '/server.py'
    print(f'{PYTHON_PATH} {file_path}')
    os.system(f'{PYTHON_PATH} {file_path}')


if __name__ == '__main__':
    main()



"""Логирование событий клиента"""

import sys
import os
sys.path.append('../')
import logging
from common.variables import LOGGING_LEVEL, ENCODING

# Объект форматирования.
client_formatter = logging.Formatter("%(asctime)s %(levelname)s %(filename)s %(message)s")


path = os.getcwd()
path = os.path.join(path, 'client.log')  # Файл для записи логов.

# Создаём потоки вывода логов.
steam = logging.StreamHandler(sys.stderr)
steam.setFormatter(client_formatter)
steam.setLevel(logging.INFO)
log_file = logging.FileHandler(path, encoding=ENCODING)
log_file.setFormatter(client_formatter)

# Создаём регистратор и настраиваем его.
logger = logging.getLogger('client')
logger.addHandler(steam)
logger.addHandler(log_file)
logger.setLevel(LOGGING_LEVEL)

if __name__ == '__main__':
    logger.critical('Тест: Критическая ошибка')
    logger.error('Тест: Ошибка')
    logger.warning('Тест: Предупреждение')
    logger.info('Тест: Информационное сообщение')
    logger.debug('Тест: Отладочная информация')



"""Логирование событий сервера"""

import sys
import os

sys.path.append('../')
import logging
from logging.handlers import TimedRotatingFileHandler
from common.variables import LOGGING_LEVEL, ENCODING

# Создаём формировщик логов (formatter):
server_formatter = logging.Formatter("%(asctime)s %(levelname)s %(module)s %(message)s")  # Объект форматирования.

# Подготовка имени файла для логирования
path = os.getcwd()
path = os.path.join(path, 'server.log')  # Файл для записи логов.
# Создаём потоки вывода логов.
steam = logging.StreamHandler(sys.stderr)
steam.setFormatter(server_formatter)
steam.setLevel(logging.INFO)
# Циклическое ежедневное логирование и переименование старых файлов в полночь по UTC.
log_file = TimedRotatingFileHandler(path,
                                    when='midnight',
                                    interval=1,
                                    backupCount=3,
                                    encoding=ENCODING,
                                    utc=True)
log_file.setFormatter(server_formatter)

# создаём регистратор и настраиваем его
logger = logging.getLogger('server')
logger.addHandler(steam)
logger.addHandler(log_file)
logger.setLevel(LOGGING_LEVEL)

if __name__ == '__main__':
    logger.critical('Тест: Критическая ошибка')
    logger.error('Тест: Ошибка')
    logger.warning('Тест: Предупреждение')  # Сообщения с важностью ниже этой не должны попасть в server.log
    logger.info('Тест: Информационное сообщение')
    logger.debug('Тест: Отладочная информация')



"""Constants"""
import logging

# Порт по умолчанию для сетевого взаимодействия.
DEFAULT_PORT = 7777
# IP-адрес по умолчанию для подключения клиента.
DEFAULT_IP_ADDRESS = '127.0.0.1'
# Max размер очереди подключений.
MAX_CONNECTION = 5
# Max длина сообщения в байтах.
MAX_PACKAGE_LENGTH = 1024
# Используемая в проекте кодировка.
ENCODING = 'utf-8'
# Установка для верхнего уровня логирования.
LOGGING_LEVEL = logging.DEBUG

# Основные ключи протокола JIM.
ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
SENDER = 'from'
DESTINATION = 'to'
DATA = 'bin'
PUBLIC_KEY = 'pubkey'

# Прочие ключи используемые в протоколе.
PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
MESSAGE = 'message'
MESSAGE_TEXT = 'mess_text'
EXIT = 'exit'
GET_CONTACTS = 'get_contacts'
LIST_INFO = 'data_list'
REMOVE_CONTACT = 'remove'
ADD_CONTACT = 'add'
USERS_REQUEST = 'get_users'
PUBLIC_KEY_REQUEST = 'pubkey_need'

# Ответы.
RESPONSE_200 = {RESPONSE: 200}
RESPONSE_202 = {RESPONSE: 202, LIST_INFO: None}
RESPONSE_205 = {RESPONSE: 205}
RESPONSE_400 = {RESPONSE: 400, ERROR: None}
RESPONSE_511 = {RESPONSE: 511, DATA: None}

# Количество попыток обращения к серверу.
ATTEMPTS = 5

HELP = f'Список поддерживаемых команд:\n' \
       f'-m, message - отправить сообщение. Для кого и текст сообщения - ввод в строке.\n' \
       f'-h, history - история сообщений.\n' \
       f'-c, contacts - список контактов.\n' \
       f'-e, edit - редактирование списка контактов.\n'\
       f'-?, help - вывести подсказки по командам.\n' \
       f'-q, exit - выход из программы.\n' \



SERVER_HELP = f'Список поддерживаемых команд:\n' \
              f'users - список известных пользователей.\n' \
              f'connected - список подключенных пользователей.\n' \
              f'loghist - история входов пользователя.\n' \
              f'exit - завершение работы сервера.\n' \
              f'help - вывод справки по поддерживаемым командам.'

SERVER_HELP_LOGHIST = f'Введите имя пользователя для просмотра истории.\n' \
                      f'Для вывода всей истории, нажмите Enter.'

SERVER_DATABASE = 'sqlite:///server_base.db3'

CLIENT_DATABASE = 'sqlite:///client.client_'



"""
Программа-лаунчер
После запуска будет выведено приглашение ввести команду.
Поддерживаемые команды:
1. s - Запустить сервер
* Запускает сервер с настройками по умолчанию.
2. k - Запустить клиенты
* Будет выведен запрос на количество тестовых клиентов для запуска.
* Клиенты будут запущены с именами вида **test1 - testX** и паролями **123**
* Тестовых пользователей необходимо предварительно, вручную зарегистрировать на сервере с паролем **123**.
* Если клиенты запускаются впервые, время запуска может быть достаточно продолжительным из-за генерации новых RSA ключей.
3. x - Закрыть все окна.
* Закрывает все активные окна, которые были запущенны из данного модуля.
4. q - Завершить работу модуля.
* Завершает работу модуля
"""

import subprocess
import sys


def main():
    processes = []
    while True:
        action = input('Выберите действие: '
                       'q - выход, '
                       's - запустить сервер, '
                       'k - запустить клиенты, '
                       'x - закрыть все окна: ')

        match action:
            case 'q':
                break
            case 's':
                # Запуск сервера.
                processes.append(subprocess.Popen(f'{sys.executable} server.py',
                                                  creationflags=subprocess.CREATE_NEW_CONSOLE))
            case 'k':
                # Запуск клиентов.
                print('Убедитесь, что на сервере зарегистрировано необходимо количество клиентов с паролем 123456.')
                print('Первый запуск может быть достаточно долгим из-за генерации ключей!')
                clients = int(input('Введите количество клиентов для запуска: '))
                for i in range(clients):
                    processes.append(subprocess.Popen(f'{sys.executable} client.py -n test{i + 1} -p 123456',
                                                      creationflags=subprocess.CREATE_NEW_CONSOLE))
            case 'x':
                while processes:
                    processes.pop().kill()


if __name__ == '__main__':
    main()



#!usr/bin/env python
from setuptools import setup, find_packages

setup(name="my_mess_proj_server",
      version="0.0.6",
      description="my_mess_proj_server",
      author="Sergey Petin",
      author_email="pillager@bk.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodomex'],
      scripts=['server/server_run.py']
      )



import argparse
import configparser
import os
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication
from common.decor import log
from common.utils import *
from server.core import MessageProcessor
from server.main_window import MainWindow
from server.database import ServerStorage

sys.path.append(os.path.join(os.getcwd(), '..'))

# Инициализация логирования сервера.
logger = logging.getLogger('server')


@log
def arg_parser(default_port, default_address):
    """
    Загрузка параметров из командной строки,
    при их отсутствии - обработка значений, принятых по умолчанию.
    """
    logger.debug(
        f'Инициализация парсера аргументов командной строки: {sys.argv}')
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', '--port', default=DEFAULT_PORT, type=int, help='Read port IP address', nargs='?')
    parser.add_argument('-a', '--addr', default='', help='Reading an IP address', nargs='?')
    parser.add_argument('-no_gui', action='store_true')
    args = parser.parse_args(sys.argv[1:])
    listen_address = args.addr
    listen_port = args.port
    gui_flag = args.no_gui
    return listen_address, listen_port, gui_flag


@log
def config_load():
    """Парсер конфигурационного ini файла."""
    config = configparser.ConfigParser()
    dir_path = os.getcwd()
    config.read(f"{dir_path}/{'server.ini'}")
    # Если конфиг файл загружен правильно, запускаемся, иначе конфиг по
    # умолчанию.
    if 'SETTINGS' in config:
        return config
    else:
        config.add_section('SETTINGS')
        config.set('SETTINGS', 'Default_port', str(DEFAULT_PORT))
        config.set('SETTINGS', 'Listen_Address', '')
        config.set('SETTINGS', 'Database_path', '')
        config.set('SETTINGS', 'Database_file', 'server_database.db3')
        return config


@log
def main():
    """Основная функция"""
    # Загрузка файла конфигурации сервера.
    config = config_load()

    # Загрузка параметров командной строки, если нет параметров, то задаём
    # значения по умолчанию.
    listen_address, listen_port, gui_flag = arg_parser(
        config['SETTINGS']['Default_port'], config['SETTINGS']['Listen_Address'])

    # Инициализация базы данных.
    database = ServerStorage(
        os.path.join(
            config['SETTINGS']['Database_path'],
            config['SETTINGS']['Database_file']))

    # Создание экземпляра класса - сервера и его запуск:
    server = MessageProcessor(listen_address, listen_port, database)
    server.daemon = True
    server.start()

    # Если указан параметр без GUI, то запускаем простенький обработчик
    # консольного ввода
    if gui_flag:
        while True:
            command = input('Введите exit для завершения работы сервера.')
            if command == 'exit':
                # Если выход, то завышаем основной цикл сервера.
                server.running = False
                server.join()
                break

    # Если не указан запуск без GUI, то запускаем GUI:
    else:
        # Создаём графическое окружение для сервера:
        server_app = QApplication(sys.argv)
        server_app.setAttribute(Qt.AA_DisableWindowContextHelpButton)
        main_window = MainWindow(database, server, config)

        # Запускаем GUI
        server_app.exec_()

        # По закрытию окон останавливаем обработчик сообщений.
        server.running = False


if __name__ == '__main__':
    main()



import json
import sys

sys.path.append('../')
from common.variables import *
from common.decor import log


@log
def get_message(sock):
    """
    Функция принимает и декодирует сообщение.
    Принимает байты, возвращает словарь,
    при несоответствии данных отдает ошибку значения.
    """

    encoded_response = sock.recv(MAX_PACKAGE_LENGTH)
    if isinstance(encoded_response, bytes):
        json_response = encoded_response.decode(ENCODING)
        response = json.loads(json_response)
        if isinstance(response, dict):
            return response
        else:
            raise TypeError


@log
def send_message(sock, message):
    """
    Функция отправки словарей через сокет.
    Функция принимает словарь, извлекает из него строку,
    строку кодирует в байты и отправляет.
    """
    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)
    sock.send(encoded_message)



"""Unit-тесты сервера"""

import sys
import os
import unittest

sys.path.append(os.path.join(os.getcwd(), '..'))
from common.variables import RESPONSE, ERROR, USER, ACCOUNT_NAME, TIME, ACTION, PRESENCE
from server import process_client_message


class TestServer(unittest.TestCase):
    """В сервере только 1 функция для тестирования"""
    err_dict = {
        RESPONSE: 400,
        ERROR: 'Bad Request'
    }
    ok_dict = {RESPONSE: 200}

    def test_ok_check(self):
        """Корректный запрос"""
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1.1, USER: {ACCOUNT_NAME: 'Guest'}}), self.ok_dict)

    def test_no_action(self):
        """Ошибка если нет действия"""
        self.assertEqual(process_client_message(
            {TIME: '1.1', USER: {ACCOUNT_NAME: 'Guest'}}), self.err_dict)

    def test_wrong_action(self):
        """Ошибка если неизвестное действие"""
        self.assertEqual(process_client_message(
            {ACTION: 'false', TIME: '1.1', USER: {ACCOUNT_NAME: 'Guest'}}), self.err_dict)

    def test_no_time(self):
        """Ошибка, если запрос не содержит штампа времени"""
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, USER: {ACCOUNT_NAME: 'Guest'}}), self.err_dict)

    def test_no_user(self):
        """Ошибка - нет пользователя"""
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: '1.1'}), self.err_dict)

    def test_unknown_user(self):
        """Ошибка - не Guest"""
        self.assertEqual(process_client_message(
            {ACTION: PRESENCE, TIME: 1.1, USER: {ACCOUNT_NAME: 'Guest1'}}), self.err_dict)

    def test_len_200_answer(self):
        """Проверка длины словаря при 200 ответе"""
        self.assertEqual(len(process_client_message(
            {ACTION: PRESENCE, TIME: 1.1, USER: {ACCOUNT_NAME: 'Guest'}})), 1)

    def test_len_400_answer(self):  # При любой ошибке должно получиться 2(тут убрал TIME)
        """Проверка длины словаря при 400 ответе"""
        self.assertEqual(len(process_client_message(
            {ACTION: PRESENCE, USER: {ACCOUNT_NAME: 'Guest'}})), 2)


if __name__ == '__main__':
    unittest.main()
